#!/bin/bash
#
# Backup Script für das sichern einer Oracle Instance
#
# Author .......... Uwe Schimanski
# Company ......... Seab@er Software AG
# Mail ............ uwe@seabaer-ag.de
# Version ......... 1.0.5
#

#+++++++++++++++++++++++++++++++++
#+++ Hier stehen die Variablen +++
#+++++++++++++++++++++++++++++++++

DATUM=`date +%Y_%m_%d`
ORACLE_SCRIPTS=/home/oracle/bin
NLS_LANG=AMERICAN_AMERICA.WE8MSWIN1252
NLS_DATE_FORMAT='DD.MM.YYYY HH24:MI:SS'
AUTHOR="Uwe Schimanski"
COMPANY="Seab@er Software AG"
MAIL="uwe@seabaer-ag.de"
WEB="http://www.seabaer-ag.de"
COPYRIGHT="© 2011 Seab@er Software AG"
VERSION=1.0.5

export ORACLE_HOME ORACLE_SCRIPTS NLS_LANG NLS_DATE_FORMAT LOG_FILE

#++++++++++++++++++++++++++++++++++++++++
#+++ Am Anfang stehen die Funktionen. +++
#++++++++++++++++++++++++++++++++++++++++

function help()
{
	clear
	printf "\n"
	printf "\033[01mNAME\033[0m\n"
	printf "\tBackupDB.sh - Backup Database mit rman.\n\n"
	printf "\033[01mSynopsis\033[0m\n"
	printf "\tBackupDB.sh -s <oracle_sid>\n"
	printf "\tBackupDB.sh -s <oracle_sid> -r\n\n"
	printf "\033[01mDESCRIPTION\033[0m\n"
	printf "\tMit diesem Script wird eine Sicherung der Datenbank durchgeführt.\n"
	printf "\tDas Oracle Home Verzeichnis wird aus der Oratab Datei ausgelesen.\n"
	printf "\tFür jede Datenbank wird automatisch eine Parameter Datei angelegt und\n"
	printf "\twenn das Log Verzeichnis nicht vorhanden ist, wird es angelegt.\n\n"
	printf "\033[01mOPTIONS\033[0m\n"
	printf "\t-s <oracle_sid>\n"
	printf "\t\tAngabe der Oracle Sid\n"
	printf "\t-r\n"
	printf "\t\tLogdatei an einer zentralen Stelle kopieren, für eine spätere Auswertung.\n"
	printf "\t-h\n"
	printf "\t\tAnzeige der Hilfe\n"
	printf "\t-i\n"
	printf "\t\tInformationen werden angezeigt\n\n"
	printf "\033[01mCOPYRIGHT\033[0m\n"
	printf "\t${COPYRIGHT}\n"
	printf "\t${WEB}\n\n"
}

function info()
{
	clear
	printf "\033[34m"
	printf "\t+++++++++++++++++++++++++++++++++++++++++++++++\n"
	printf "\t+                                             +\n"
	printf "\t+ Author ......... ${AUTHOR}             +\n"
	printf "\t+ Company ........ ${COMPANY}        +\n" 
	printf "\t+ Mail ........... ${MAIL}          +\n"
	printf "\t+ Web ............ ${WEB}   +\n"
	printf "\t+ Copyright ...... ${COPYRIGHT} +\n"
	printf "\t+ Version ........ ${VERSION}                      +\n"
	printf "\t+                                             +\n"
	printf "\t+++++++++++++++++++++++++++++++++++++++++++++++\n\n"
	printf "\033[0m"
}

function log_header()
{
	printf "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n" > ${LOG_FILE}
	printf "+\n" >> ${LOG_FILE}
	printf "+ Backup der Datenbank ${ORACLE_SID}.\n" >> ${LOG_FILE}
	printf "+\n" >> ${LOG_FILE}
	printf "+ Author ............... ${AUTHOR}\n" >> ${LOG_FILE}
	printf "+ Company .............. ${COMPANY}\n" >> ${LOG_FILE}
	printf "+ Mail ................. ${WEB}\n" >> ${LOG_FILE}
	printf "+ Version .............. ${VERSION}\n" >> ${LOG_FILE}
	printf "+\n" >> ${LOG_FILE}
	printf "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n" >> ${LOG_FILE}
	printf "+ Script Variablen\n" >> ${LOG_FILE}
	printf "+ =================\n" >> ${LOG_FILE}
	printf "+ ORACLE_HOME     = ${ORACLE_HOME}\n" >> ${LOG_FILE}
	printf "+ ORACLE_SCRIPTS  = ${ORACLE_SCRIPTS}\n" >> ${LOG_FILE}
	printf "+ NLS_LANG        = ${NLS_LANG}\n" >> ${LOG_FILE}
	printf "+ NLS_DATE_FORMAT = ${NLS_DATE_FORMAT}\n" >> ${LOG_FILE}
	printf "+ LOG_RMAN        = ${LOG_RMAN}\n" >> ${LOG_FILE}
	printf "+ LOG_FILE        = ${LOG_FILE}\n" >> ${LOG_FILE}
	printf "+ BACKUP_PATH     = ${BACKUP_PATH}\n" >> ${LOG_FILE}
	printf "+ ORACLESID       = ${ORACLESID}\n" >> ${LOG_FILE}
	printf "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n\n" >> ${LOG_FILE}
}

function logpath()
{
	if [ ! -d "/home/oracle/log/${ORACLESID}" ];
	then
		mkdir -p /home/oracle/log/${ORACLESID}
		printf "\n"
		printf "++++++++++++++++++++++++++++++++++++++++++++++++++\n"
		printf "+\n"
		printf "+ Log Path /home/oracle/log/${ORACLESID} wurde angelegt.\n"
		printf "+\n"
		printf "++++++++++++++++++++++++++++++++++++++++++++++++++\n\n"
	fi
}

function backup_par()
{
	if [ ! -r "/home/oracle/bin/${ORACLESID}.par" ];
	then
		PAR_FILE=${ORACLE_SCRIPTS}/${ORACLESID}.par
		cat <<-EOF > ${PAR_FILE}
		connect target /
		backup incremental level=0 database tag level0 plus archivelog delete all input;
		sql "alter database backup controlfile to trace as ''/u01/app/oracle/admin/${ORACLESID}/backup/crctl.sql'' reuse";
		crosscheck backup;
		delete obsolete;
		delete expired backup;
		EOF
		printf "\n"
		printf "++++++++++++++++++++++++++++++++++++++++++++++++++\n"
		printf "+\n"
		printf "+ Der Rman Parameterfile ${PAR_FILE} wurde angelegt.\n"
		printf "+\n"
		printf "++++++++++++++++++++++++++++++++++++++++++++++++++\n\n"
	fi
}

function  backup_db()
{
	if [ ! -d ${BACKUP_PATH} ];
	then
		mkdir ${BACKUP_PATH}
	fi

	${ORACLE_HOME}/bin/rman cmdfile=${ORACLE_SCRIPTS}/${ORACLESID}.par log=${LOG_RMAN}
	cat ${LOG_RMAN} >> ${LOG_FILE}
	rm ${LOG_RMAN}
}

function reporting()
{
	if  test -r ${LOG_FILE} 
	then
		cp ${LOG_FILE} ${REPORTING_PATH}/${DATUM}_Level0.log
		printf "Backup erfolgt" > ${REPORTING_PATH}/Backup.txt
	fi
}

#++++++++++++++++++++++++++++++++
#+++ Auswertung der Parameter +++
#++++++++++++++++++++++++++++++++

while getopts "hirs:" OPT; do
case ${OPT} in
	h)
		help
		;;
	i)
		info
		;;
	r)
		REPORT=true
		;;
	s)
		BACKUP=true
		ORACLESID=${OPTARG}
		ORACLE_SID=${ORACLESID}
		LOG_RMAN=/home/oracle/log/${ORACLESID}/Level0.log
		LOG_FILE=/home/oracle/log/${ORACLESID}/${DATUM}_Level0.log
		BACKUP_PATH=/u01/app/oracle/admin/${ORACLESID}/backup
		REPORTING_PATH=/filerged/rman_log/${ORACLESID}
		export ORACLE_SID
		;;
	?)	# Für den Rest
		help
		;;
esac
done

#+++++++++++++++++++++
#+++ Main Programm +++
#+++++++++++++++++++++

if [ -n "${BACKUP}" ];
then
	if [ -r /etc/oratab ];
	then
		ORACLE_HOME=`awk -F: '/^[^#].*:.+:[YN]/ {if ($2!="") print $2; exit}' </etc/oratab`
		OSID=`awk -F: '/^[^#].*:.+:[YN]/ {if ($1!="") print $1; exit}' </etc/oratab`
		if [ "${OSID}" = "${ORACLESID}" ];
		then
			logpath
			log_header
			backup_par
			backup_db
		else
			printf "\n\033[31m\033[01m\033[05m  ACHTUNG!\033[0m\n"
			printf "\n\033[31m  Die eingegebene Oracle SID: ${ORACLESID} konnte in der oratab nicht gefunden werden!\n"
			printf "\n  In der Oratab steht folgende Oracle SID: ${OSID}.\033[0m\n\n"
		fi
		if [  -n "${REPORT}" ];
		then
			reporting
		fi
	else
		printf "\n\033[31m\033[01m\033[05m  ACHTUNG!\033[0m\n"
		printf "\n\033[31m  Es konnte kein Oracle Home gefunden werden.\033[0m\n\n"
	fi
fi

# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# +++ Hilfe anzeigen, wenn nur das Programm aufgerufen wird. +++
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

if [ -z "$1" ];
then
	help
fi
